package linkMRM.ExFrameSample;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Scanner;

import GenCol.entity;
import linkMRM.flitSuper;
import model.modeling.message;
import view.modeling.ViewableAtomic;

public class Transducer extends ViewableAtomic{

	String folderName;
	ArrayList<outputData> data = new ArrayList<outputData>();

	public Transducer(String folderName) {
		super("transducer");
		this.folderName = folderName;
		
		addInport("channelOut");
		readFile();
	}
	
	/*initialization*/
	public void initialize( ){
		phase = "passive";
		sigma = INFINITY;
		super.initialize( );
	}
	
	/*external Transition Function*/
	public void  deltext(double e, message m){
		Continue(e);
		for(int i = 0 ; i < m.size() ; i++){
			if(messageOnPort(m, "channelOut", i)){
				flitSuper travellingFlit = (flitSuper) (m.getValOnPort("channelOut", i));
				checkValidity(this.getSimulator().getTL() + e, "channelOut", travellingFlit);
			}
		}
	}
	
	private void checkValidity(double time, String port, flitSuper travellingFlit) {
		outputData thisData = data.get(0);
		if(round(thisData.time, 2) == round(time, 2)){
			if(travellingFlit.equals(thisData.output)){
				System.out.println("Correct: event received at time " + time + " on port " + port);
			}
			else{
				System.out.println("ERROR: event received at the right time (" + time + 
						") with incorrect values");
			}
			data.remove(0);
		}
		
		else if(thisData.time > time){
			System.out.println("ERROR: unexpected event received at " + time + " on port " + port);
		}
		
		else {
			System.out.println("ERROR: event missed at time " + thisData.time + " on port " + thisData.portName);
			data.remove(0);
		}
	}

	protected void readFile() {
		File fileOutputs = new File(folderName + "/outputs.txt");
		Scanner reader = null;
		try {
			reader = new Scanner(fileOutputs);
		} catch (FileNotFoundException e) {
			System.out.println("Failure: Cannot read inputs file");
			e.printStackTrace();
			return ;
		}		

		
		while(reader.hasNextLine()){
			Double time = Double.parseDouble(reader.nextLine());
			
			String portName = null;
			if(reader.hasNextLine())
				portName = reader.nextLine();
			else continue;
			
			entity input = null;
						
			try {
				input = (entity)deserializeObject(reader.nextLine());
			} catch (ClassNotFoundException | IOException e) {
				System.out.println("Failure - deserialization incomplete");
				e.printStackTrace();
				continue;
			}
					
			System.out.println(portName + " "  + time);
			
			this.data.add(new outputData(portName, time, input));
		}
	}
	
	protected Object deserializeObject(String objectString) throws IOException ,
	ClassNotFoundException {
		byte [] data = Base64.getDecoder().decode(objectString);
		ObjectInputStream ois = new ObjectInputStream( 
				new ByteArrayInputStream(data));
		Object o  = ois.readObject();
		ois.close();
		return o;
	}

	protected class outputData {
		String portName;
		double time;
		entity output;
		
		public outputData(String portname, double time, entity output){
			this.portName = portname;
			this.time = time;
			this.output = output;
		}
	}
	
	private double round(double value, int places) {
	    if (places < 0) throw new IllegalArgumentException();

	    BigDecimal bd = new BigDecimal(value);
	    bd = bd.setScale(places, RoundingMode.HALF_UP);
	    return bd.doubleValue();
	}
}
