package linkMRM.L2;

import java.awt.Color;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Base64;

import linkMRM.flitSuper;
import model.modeling.message;
import view.modeling.ViewableAtomic;

public class Channel extends ViewableAtomic{
	protected double channelDelay;
	protected flitSuper travellingFlit;
	protected FileWriter inputsWriter;
	protected FileWriter outputsWriter;

	public Channel() {this("Channel", .5);}
	public Channel(String name, double delay){
		super(name);

		addInport("channelIn");
		addInport("send");
		addInport("failStopSignal");
		addOutport("channelOut");

		this.channelDelay = delay;
		
		this.backgroundColor = Color.PINK;

		File fileInputs = new File(Config.experimentFolder + this.name + "/inputs.txt");
		File fileOutputs = new File(Config.experimentFolder + this.name + "/outputs.txt");

		try {
			new File(Config.experimentFolder + this.name).mkdir();
			inputsWriter = new FileWriter(fileInputs);
			outputsWriter = new FileWriter(fileOutputs);
		} catch (IOException e) {
			e.printStackTrace();
		}	
	}

	protected void finalize() throws Throwable {
		  super.finalize();

		  inputsWriter.flush();
		  inputsWriter.close();
		  
		  outputsWriter.flush();
		  outputsWriter.close();
	}
	
	/*initialization*/
	public void initialize( ){
		phase = "passive";
		sigma = INFINITY;
		super.initialize( );
	}

	/*External Transition Function*/
	public void  deltext(double e, message x){
		Continue(e);
		if(!phaseIs("stop")){
			for(int i = 0 ; i < x.size() ; i++){
				if(messageOnPort(x, "channelIn", i)){
					travellingFlit = (flitSuper) (x.getValOnPort("channelIn", i));
					holdIn("active", channelDelay);
					recordExperimentInput("channelIn", this.getSimulator().getTL() + e, travellingFlit);
				}

				else if(messageOnPort(x, "send", i)){
					holdIn("active", channelDelay);
					recordExperimentInput("send", this.getSimulator().getTL() + e, 
							((Signal) (x.getValOnPort("send", i))));
				}

				else if(messageOnPort(x, "failStopSignal", i)){
					holdIn("stop", INFINITY);
					recordExperimentInput("failStopSignal", this.getSimulator().getTL() + e, 
							((Signal) (x.getValOnPort("failStopSignal", i))));
				}
			}
		}
	}

	/*Internal Transition Function*/
	public void  deltint( ){
		if(phaseIs("active")){
			passivate();
		}
	}

	/*Output Function*/
	public message out(){
		message m = new message();
		if ((phaseIs("active"))){	  
			m.add(makeContent("channelOut", travellingFlit));
			recordExperimentOutput("channelOut", this.getSimulator().getTN(), travellingFlit);
		}  
		return m;
	}
	
	protected void recordExperimentInput(String port, double time, Serializable value){
		String stringVal;
		try {
			stringVal = serializeObject(value);
			inputsWriter.write(time + "\n" + port + "\n" + stringVal + "\n");
			inputsWriter.flush();
		} catch (IOException e) {
			System.out.println("Error writing experiments - inputs");
			e.printStackTrace();
		}
	}
	
	protected void recordExperimentOutput(String port, double time, Serializable value){
		try {
			String stringVal = serializeObject(value);
			outputsWriter.write(time + "\n" + port + "\n" + stringVal + "\n");
			outputsWriter.flush();
		} catch (IOException e) {
			System.out.println("Error writing experiments - outputs");
			e.printStackTrace();
		}
	}
	
	protected String serializeObject(Serializable o) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream( baos );
        oos.writeObject( o );
        oos.close();
        return Base64.getEncoder().encodeToString(baos.toByteArray());
	}
	
}
