package linkMRM.L2;

import java.awt.Color;

import linkMRM.flitSuper;
import model.modeling.message;
import view.modeling.ViewableAtomic;

public class Transducer extends ViewableAtomic{

	flitSuper generatedFlit;
	double receivingTime; 
	boolean experimentPassed = true;
	protected double linkDelay;
	double timePast = 0; 
	
	public Transducer(double linkDelay){
		this("Transducer", linkDelay);
	}
	
	public Transducer(String name, double linkDelay){
		super(name);
		
		addInport("ackFromLink");
		addInport("fromLink");
		addInport("fromGen");
		
		this.linkDelay = linkDelay;
		
		this.backgroundColor = Color.GREEN;

	}
	
	/*initialization*/
	public void initialize(){
		phase = "passive";
		sigma = INFINITY;
		super.initialize();
	}
	

	/*External Transition Function*/
	public void  deltext(double e, message x){
		Continue(e);
		timePast += e;
		for(int i = 0 ; i < x.size() ; i++){
			if(messageOnPort(x, "fromLink", i)){
				flitSuper linkFlit = (flitSuper)x.getValOnPort("fromLink", i);
				double time = timePast;
				double difference = time - receivingTime;

				if(difference % linkDelay != 0){
					System.out.println("Failure: timing error (Difference is " + difference + ")");
				}
				
				if(!linkFlit.equals(generatedFlit)){
					System.out.println("Failure: flit data error");
				}
				
				experimentPassed = true; 
			}
			else if(messageOnPort(x, "fromGen", i)){
				if(!experimentPassed){
					System.out.println("WARNING: packet override detected!");
				}
				generatedFlit = (flitSuper)x.getValOnPort("fromGen", i);
				receivingTime = timePast;
				experimentPassed = false;
			}
		}
		
		if(!checkFlitAndAckRelationship(x)){
			System.out.println("Failure: inconsistent ack and flit received");
		}
	}
	
	public void deltint(){
		passivate();
	}
	
	public message out(){
		message m = new message();
		return m;
	}
	
	private boolean checkFlitAndAckRelationship(message x){
		int numOfAcks = 0;
		int numOfFlits = 0;
		
		for(int i = 0 ; i < x.size() ; i++){
			if(messageOnPort(x, "fromLink", i))
				numOfFlits++;
			else if(messageOnPort(x, "ackFromLink", i))
				numOfAcks++;
		}
		
		if((numOfAcks == 1 && numOfFlits == 1) || (numOfAcks == 0 && numOfFlits == 0))
			return true;
		return false; 
	}
}

